DLL {
	global:
		Kinematics_C;
		DiffKinematics_C;
		Velocities_C;
		I_I_C;
		Accelerations_C;
		ID_C;
		FD_C;
		SPART_SPACEROBOT_ODE_C;
		SPART_C_initialize;
		SPART_C_terminate;
		emxInit_cell_wrap_0;
		emxEnsureCapacity_cell_wrap_0;
		emxFree_cell_wrap_0;
		emxEnsureCapacity_cell_wrap_2;
		emxEnsureCapacity_cell_wrap_1;
		emxFree_cell_wrap_2;
		emxInit_cell_wrap_3;
		emxEnsureCapacity_cell_wrap_3;
		emxFree_cell_wrap_3;
		emxFree_cell_wrap_1;
		emxEnsureCapacity_real_T;
		emxFree_struct1_T;
		emxFree_struct0_T;
		emxFree_real_T;
		emxInit_real_T;
		emxInit_cell_wrap_1;
		emxInit_cell_wrap_2;
		emxInit_struct0_T;
		emxInit_struct1_T;
		emxInitArray_cell_wrap_1;
		emxInitArray_cell_wrap_2;
		emxCreate_cell_wrap_2;
		emxCreate_cell_wrap_1;
		emxCreateND_cell_wrap_1;
		emxDestroyArray_cell_wrap_1;
		emxDestroyArray_cell_wrap_2;
		emxCreate_struct0_T;
		emxCreate_struct1_T;
		emxCreate_real_T;
		emxCreateND_real_T;
		emxInitArray_real_T;
		emxDestroyArray_struct1_T;
		emxDestroyArray_struct0_T;
		emxDestroyArray_real_T;
		emxCreateWrapper_real_T;
		emxCreateWrapperND_real_T;
		emxCreateWrapper_struct0_T;
		emxCreateND_struct0_T;
		emxCreateWrapperND_struct0_T;
		emxCreateWrapper_struct1_T;
		emxCreateND_struct1_T;
		emxCreateWrapperND_struct1_T;
		emxCreateWrapper_cell_wrap_1;
		emxCreateWrapperND_cell_wrap_1;
		emxCreateWrapper_cell_wrap_2;
		emxCreateND_cell_wrap_2;
		emxCreateWrapperND_cell_wrap_2;
		emxInitArray_struct0_T;
		emxInitArray_struct1_T;
	local:
		*;
};
