/*
 * Academic License - for use in teaching, academic research, and meeting
 * course requirements at degree granting institutions only.  Not for
 * government, commercial, or other organizational use.
 * File: SPART_C_emxAPI.h
 *
 * MATLAB Coder version            : 25.1
 * C/C++ source code generated on  : 24-Feb-2026 15:36:29
 */

#ifndef SPART_C_EMXAPI_H
#define SPART_C_EMXAPI_H

/* Include Files */
#include "SPART_C_types.h"
#include "rtwtypes.h"
#include <stddef.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Function Declarations */
extern emxArray_cell_wrap_1 *emxCreateND_cell_wrap_1(int numDimensions,
                                                     const int *size);

extern emxArray_cell_wrap_2 *emxCreateND_cell_wrap_2(int numDimensions,
                                                     const int *size);

extern emxArray_real_T *emxCreateND_real_T(int numDimensions, const int *size);

extern emxArray_struct0_T *emxCreateND_struct0_T(int numDimensions,
                                                 const int *size);

extern emxArray_struct1_T *emxCreateND_struct1_T(int numDimensions,
                                                 const int *size);

extern emxArray_cell_wrap_1 *emxCreateWrapperND_cell_wrap_1(cell_wrap_1 *data,
                                                            int numDimensions,
                                                            const int *size);

extern emxArray_cell_wrap_2 *emxCreateWrapperND_cell_wrap_2(cell_wrap_2 *data,
                                                            int numDimensions,
                                                            const int *size);

extern emxArray_real_T *
emxCreateWrapperND_real_T(double *data, int numDimensions, const int *size);

extern emxArray_struct0_T *emxCreateWrapperND_struct0_T(struct0_T *data,
                                                        int numDimensions,
                                                        const int *size);

extern emxArray_struct1_T *emxCreateWrapperND_struct1_T(struct1_T *data,
                                                        int numDimensions,
                                                        const int *size);

extern emxArray_cell_wrap_1 *emxCreateWrapper_cell_wrap_1(cell_wrap_1 *data,
                                                          int rows, int cols);

extern emxArray_cell_wrap_2 *emxCreateWrapper_cell_wrap_2(cell_wrap_2 *data,
                                                          int rows, int cols);

extern emxArray_real_T *emxCreateWrapper_real_T(double *data, int rows,
                                                int cols);

extern emxArray_struct0_T *emxCreateWrapper_struct0_T(struct0_T *data, int rows,
                                                      int cols);

extern emxArray_struct1_T *emxCreateWrapper_struct1_T(struct1_T *data, int rows,
                                                      int cols);

extern emxArray_cell_wrap_1 *emxCreate_cell_wrap_1(int rows, int cols);

extern emxArray_cell_wrap_2 *emxCreate_cell_wrap_2(int rows, int cols);

extern emxArray_real_T *emxCreate_real_T(int rows, int cols);

extern emxArray_struct0_T *emxCreate_struct0_T(int rows, int cols);

extern emxArray_struct1_T *emxCreate_struct1_T(int rows, int cols);

extern void emxDestroyArray_cell_wrap_1(emxArray_cell_wrap_1 *emxArray);

extern void emxDestroyArray_cell_wrap_2(emxArray_cell_wrap_2 *emxArray);

extern void emxDestroyArray_real_T(emxArray_real_T *emxArray);

extern void emxDestroyArray_struct0_T(emxArray_struct0_T *emxArray);

extern void emxDestroyArray_struct1_T(emxArray_struct1_T *emxArray);

extern void emxInitArray_cell_wrap_1(emxArray_cell_wrap_1 **pEmxArray,
                                     int numDimensions);

extern void emxInitArray_cell_wrap_2(emxArray_cell_wrap_2 **pEmxArray,
                                     int numDimensions);

extern void emxInitArray_real_T(emxArray_real_T **pEmxArray, int numDimensions);

extern void emxInitArray_struct0_T(emxArray_struct0_T **pEmxArray,
                                   int numDimensions);

extern void emxInitArray_struct1_T(emxArray_struct1_T **pEmxArray,
                                   int numDimensions);

#ifdef __cplusplus
}
#endif

#endif
/*
 * File trailer for SPART_C_emxAPI.h
 *
 * [EOF]
 */
