/*
 * Academic License - for use in teaching, academic research, and meeting
 * course requirements at degree granting institutions only.  Not for
 * government, commercial, or other organizational use.
 * File: SPART_C_data.c
 *
 * MATLAB Coder version            : 25.1
 * C/C++ source code generated on  : 24-Feb-2026 15:36:29
 */

/* Include Files */
#include "SPART_C_data.h"

/* Variable Definitions */
omp_nest_lock_t SPART_C_nestLockGlobal;

const signed char iv[9] = {1, 0, 0, 0, 1, 0, 0, 0, 1};

boolean_T isInitialized_SPART_C = false;

/*
 * File trailer for SPART_C_data.c
 *
 * [EOF]
 */
